<?php
/**
 * Copyright (C) 2017-2018 thirty bees
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.md
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@thirtybees.com so we can send you a copy immediately.
 *
 *  @author    thirty bees <modules@thirtybees.com>
 *  @copyright 2017-2018 thirty bees
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */

namespace VATNumberModule;

if (!defined('_TB_VERSION_')) {
    exit;
}

require_once __DIR__.'/../vendor/autoload.php';

/**
 * Class VatStatus
 */
class VatStatus extends \ObjectModel
{
    const PRIMARY = 'id_vat_number_status';
    const TABLE = 'vat_number_status';

    // @codingStandardsIgnoreStart
    /** @var int $id_customer */
    public $id_customer;
    /** @var string $vat_number */
    public $vat_number;
    /** @var bool $valid */
    public $valid;
    /** @var bool $manual */
    public $manual;
    /** @var string $vat_company */
    public $vat_company;
    /** @var string $vat_address */
    public $vat_address;
    // @codingStandardsIgnoreEnd

    /**
     * @see ObjectModel::$definition
     */
    public static $definition = [
        'table' => self::TABLE,
        'primary' => self::PRIMARY,
        'fields' => [
            'id_customer'  => ['type' => self::TYPE_INT,    'validate' => 'isUnsignedId', 'db_type' => 'INT(11) UNSIGNED'   ],
            'vat_number'   => ['type' => self::TYPE_STRING, 'validate' => 'isString',     'db_type' => 'VARCHAR(128)'       ],
            'valid'        => ['type' => self::TYPE_BOOL,   'validate' => 'isBool',       'db_type' => 'TINYINT(1) UNSIGNED'],
            'manual'       => ['type' => self::TYPE_BOOL,   'validate' => 'isBool',       'db_type' => 'TINYINT(1) UNSIGNED'],
            'vat_company'  => ['type' => self::TYPE_STRING, 'validate' => 'isString',     'db_type' => 'VARCHAR(128)'       ],
            'vat_address'  => ['type' => self::TYPE_STRING, 'validate' => 'isString',     'db_type' => 'VARCHAR(255)'       ],
            'date_add'     => ['type' => self::TYPE_DATE,   'validate' => 'isDate',       'db_type' => 'DATETIME'           ],
            'date_upd'     => ['type' => self::TYPE_DATE,   'validate' => 'isDate',       'db_type' => 'DATETIME'           ],
        ],
    ];

    /**
     * Get by Customer ID
     *
     * @param int $idCustomer
     *
     * @return static
     * @throws \Adapter_Exception
     * @throws \PrestaShopDatabaseException
     * @throws \PrestaShopException
     */
    public static function getByCustomerId($idCustomer)
    {
        $result = \Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow(
            (new \DbQuery())
                ->select('*')
                ->from(static::TABLE)
                ->where('`id_customer` = '.(int) $idCustomer)

        );

        $vatStatus = new static();
        if ($result) {
            $vatStatus->hydrate($result);
        }

        return $vatStatus;
    }

    /**
     * Assign the Customer Group
     *
     * @throws \PrestaShopException
     *
     * @return bool
     */
    public function assignGroup()
    {
        $groups = json_decode(\Configuration::get(\VatNumber::GROUPS_CONFIG), true);
        $prefix = strtoupper(substr($this->vat_number, 0, 2));
        if (!empty($groups) && !empty($groups[$prefix])) {
            $idGroup = $groups[$prefix];
            $customer = new \Customer($this->id_customer);
            if ($customer->id_default_group == $idGroup) {
                return true;
            }

            $customer->id_default_group = (int) $idGroup;
            $customer->addGroups([$idGroup]);

            return $customer->save();
        }

        return true;
    }
}
