<?php
/**
 * Copyright (C) 2017-2018 thirty bees
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.md
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@thirtybees.com so we can send you a copy immediately.
 *
 * @author    thirty bees <contact@thirtybees.com>
 * @copyright 2017-2018 thirty bees
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */

use VATNumberModule\VatStatus;
use VATNumberModule\VatTools;

if (!defined('_TB_VERSION_')) {
    return;
}

require_once __DIR__.'/../../vatnumber.php';

/**
 * Class AdminPostNLExportController
 *
 * @since 1.0.0
 *
 * @property VATNumber $module
 */
class AdminVatNumbersController extends ModuleAdminController
{
    // @codingStandardsIgnoreStart
    /** @var string $toolbar_title */
    public $toolbar_title;
    /** @var array $statuses_array */
    protected $statuses_array = [];
    // @codingStandardsIgnoreEnd

    /**
     * AdminVatNumbersController constructor.
     *
     * @throws PrestaShopException
     */
    public function __construct()
    {

        $this->bootstrap = true;
        $this->table = 'customer';
        $this->className = 'Customer';
        $this->lang = false;
        $this->addRowAction('view');
        $this->explicitSelect = true;
        $this->allow_export = true;
        $this->deleted = false;
        $this->context = Context::getContext();

        $this->_select = 'a.`id_customer`, a.`firstname`, a.`lastname`, a.`email`, a.`company`, IFNULL(v.`vat_number`, \'--\') AS `vat_number`, \'--\' as `id_address`';
        $this->_select .= ', v.`valid`, v.`manual`, v.`vat_company`, IFNULL(v.`vat_address`, \'--\') AS `vat_address`';
        $this->_join = 'LEFT JOIN `'._DB_PREFIX_.VatStatus::TABLE."` v ON v.`id_customer` = a.`id_customer` \n";
        $this->_orderBy = 'id_customer';
        $this->_orderWay = 'DESC';

        $this->_use_found_rows = true;

        $this->fields_list =
            [
                'id_customer' => [
                    'title'      => $this->l('ID'),
                    'align'      => 'text-center',
                    'class'      => 'fixed-width-xs',
                    'filter_key' => 'a!id_customer',
                ],
                'firstname'   => [
                    'title'      => $this->l('First name'),
                    'filter_key' => 'a!firstname',
                ],
                'lastname'    => [
                    'title'      => $this->l('Last name'),
                    'filter_key' => 'a!lastname',
                ],
                'email'       => [
                    'title'      => $this->l('Email address'),
                    'filter_key' => 'a!email',
                ],
                'company'     => [
                    'title'      => $this->l('Company'),
                    'filter_key' => 'a!company',
                ],
                'vat_number'  => [
                    'title'          => $this->l('VAT Number'),
                    'callback'       => 'displayVatNumber',
                    'remove_onclick' => true,
                    'type' => 'select',
                    'list' => [
                        0 => 'No',
                        1 => 'Yes',
                    ],
                    'filter_key' => 'v!vat_number'
                ],
                'id_address'  => [
                    'title'          => $this->l('Address'),
                    'callback'       => 'displayAddress',
                    'remove_onclick' => true,
                    'search'         => false,
                    'orderby'        => false,
                ],
                'vat_address' => [
                    'title'          => $this->l('VAT Number address'),
                    'callback'       => 'displayVatCompany',
                    'remove_onclick' => true,
                ],
            ];

        $this->shopLinkType = 'shop';
        $this->shopShareDatas = Shop::SHARE_CUSTOMER;

        $this->bulk_actions = [];

        parent::__construct();
    }

    /**
     * Init controller
     *
     * @throws Exception
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     * @throws SmartyException
     */
    public function init()
    {
        if (Tools::isSubmit('viewcustomer')) {
            $newToken = Tools::getAdminTokenLite('AdminCustomers');

            $requestUri = str_replace('AdminVatNumbers', 'AdminCustomers', $_SERVER['REQUEST_URI']);
            $requestUri = preg_replace("/^(.*)?(&|\?)(token=[a-z0-9]{32})(.*)$/", "$1$2token=$newToken$4", $requestUri);

            Tools::redirectAdmin($requestUri);
        }

        parent::init();
    }

    /**
     * @throws Exception
     * @throws PrestaShopException
     * @throws SmartyException
     */
    public function initContent()
    {
        if (!Configuration::get('PS_B2B_ENABLE')) {
            return;
        }

        parent::initContent();
    }

    /**
     * @return void
     *
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public function setMedia()
    {
        Media::addJsDef([
            'vat_number_ajax' => $this->context->link->getAdminLink('AdminVatNumbers', true),
        ]);

        parent::setMedia();
        $this->addJS(_PS_MODULE_DIR_.'vatnumber/views/js/vatnumber.js');
    }

    /**
     * @return void
     *
     * @throws Adapter_Exception
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public function ajaxProcessValidateVatNumber()
    {
        $vatNumber = Tools::getValue('vatNumber');
        $idCustomer = Tools::getValue('idCustomer');

        $check = VatNumber::viesCheck($vatNumber);
        if (is_array($check) && count($check)) {
            $check = array_values($check)[0];
            if ($check['valid']) {
                $status = VatStatus::getByCustomerId($idCustomer);
                $status->id_customer = (int) $idCustomer;
                $status->valid = true;
                $status->manual = false;
                $status->vat_number = $vatNumber;
                $status->vat_company = $check['traderName'];
                $status->vat_address = $check['traderAddress'];

                if ($status->save()) {
                    $status->assignGroup();
                }
            }
        }
        header('Content-Type: application/json; charset=utf-8');
        $this->ajaxDie(json_encode([
            'success' => true,
            'check'   => $check,
        ]));
    }

    /**
     * @return void
     *
     * @throws Adapter_Exception
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public function ajaxProcessValidateAndSetVatNumber()
    {
        $vatNumber = Tools::getValue('vatNumber');
        $idCustomer = Tools::getValue('idCustomer');

        $check = VatNumber::viesCheck($vatNumber);
        if (is_array($check) && count($check)) {
            $check = array_values($check)[0];
            if ($check['valid']) {
                $status = VatStatus::getByCustomerId($idCustomer);
                $status->id_customer = (int) $idCustomer;
                $status->valid = true;
                $status->manual = false;
                $status->vat_number = $vatNumber;
                $status->vat_company = $check['traderName'];
                $status->vat_address = $check['traderAddress'];

                if ($status->save()) {
                    $status->assignGroup();

                    $address = new Address(Address::getFirstCustomerAddressId($idCustomer));
                    $address->vat_number = $vatNumber;
                    $address->save();
                }
            }
        }
        header('Content-Type: application/json; charset=utf-8');
        if (!is_array($check)) {
            $this->ajaxDie(json_encode([
                'success' => false,
                'valid'   => false,
                'message' => $this->module->l('Invalid VAT Number'),
            ]));
        }
        $this->ajaxDie(json_encode(
            [
                'success' => true,
                'check'   => $check,
            ]
        ));
    }

    /**
     * @param int   $id
     * @param array $tr
     *
     * @return string
     * @throws Exception
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     * @throws ReflectionException
     * @throws SmartyException
     */
    public function displayVatNumber($id, $tr)
    {
        $reflection = new ReflectionClass($this->module);
        $vatNumber = $tr['vat_number'];
        if (!$vatNumber || $vatNumber === '--') {
            $vatNumber = '';
            $customer = new Customer($tr['id_customer']);
            $addresses = $customer->getAddresses($this->context->language->id);
            foreach ($addresses as $address) {
                if ($address['vat_number']) {
                    $vatNumber = $address['vat_number'];
                    break;
                }
            }
        }

        $this->context->smarty->assign([
            'id'        => $id,
            'tr'        => $tr,
            'vatNumber' => $vatNumber,
            'manual'    => (bool) $tr['manual'],
            'valid'     => (bool) $tr['valid'],
        ]);

        if ($vatNumber) {
            return $this->module->display($reflection->getFileName(), 'views/templates/admin/listvatnumber.tpl');
        }

        return '--';
    }

    /**
     * @param int   $id
     * @param array $tr
     *
     * @return string
     * @throws Exception
     * @throws SmartyException
     */
    public function displayVatCompany($id, $tr)
    {
        $reflection = new ReflectionClass($this->module);

        $this->context->smarty->assign([
            'id' => $id,
            'tr' => $tr,
        ]);

        return $this->module->display($reflection->getFilename(), 'views/templates/admin/listvatcompany.tpl');
    }

    /**
     * @param int   $id
     * @param array $tr
     *
     * @return string
     * @throws Exception
     * @throws SmartyException
     */
    public function displayAddress($id, $tr)
    {
        $reflection = new ReflectionClass($this->module);

        $this->context->smarty->assign([
            'id'           => $id,
            'tr'           => $tr,
            'address'      => new Address(Address::getFirstCustomerAddressId($tr['id_customer'], true)),
            'addressCount' => VatTools::customerAddressCount($tr['id_customer']),
        ]);

        return $this->module->display($reflection->getFilename(), 'views/templates/admin/listaddress.tpl');
    }

    /**
     * Get the current objects' list form the database
     *
     * @param int         $idLang   Language used for display
     * @param string|null $orderBy  ORDER BY clause
     * @param string|null $orderWay Order way (ASC, DESC)
     * @param int         $start    Offset in LIMIT clause
     * @param int|null    $limit    Row count in LIMIT clause
     * @param int|bool    $idLangShop
     *
     * @throws \PrestaShopDatabaseExceptionCore
     * @throws \PrestaShopExceptionCore
     *
     * @since   1.0.0
     * @version 1.0.0 Initial version
     */
    public function getList(
        $idLang,
        $orderBy = null,
        $orderWay = null,
        $start = 0,
        $limit = null,
        $idLangShop = false
    ) {
        $this->dispatchFieldsListingModifierEvent();

        $this->ensureListIdDefinition();

        /* Manage default params values */
        $useLimit = true;
        if ($limit === false) {
            $useLimit = false;
        } elseif (empty($limit)) {
            if (isset($this->context->cookie->{$this->list_id.'_pagination'}) && $this->context->cookie->{$this->list_id.'_pagination'}) {
                $limit = $this->context->cookie->{$this->list_id.'_pagination'};
            } else {
                $limit = $this->_default_pagination;
            }
        }

        if (!Validate::isTableOrIdentifier($this->table)) {
            throw new PrestaShopException(sprintf('Table name %s is invalid:', $this->table));
        }
        $prefix = str_replace(['admin', 'controller'], '', mb_strtolower(get_class($this)));
        if (empty($orderBy)) {
            if ($this->context->cookie->{$prefix.$this->list_id.'Orderby'}) {
                $orderBy = $this->context->cookie->{$prefix.$this->list_id.'Orderby'};
            } elseif ($this->_orderBy) {
                $orderBy = $this->_orderBy;
            } else {
                $orderBy = $this->_defaultOrderBy;
            }
        }

        if (empty($orderWay)) {
            if ($this->context->cookie->{$prefix.$this->list_id.'Orderway'}) {
                $orderWay = $this->context->cookie->{$prefix.$this->list_id.'Orderway'};
            } elseif ($this->_orderWay) {
                $orderWay = $this->_orderWay;
            } else {
                $orderWay = $this->_defaultOrderWay;
            }
        }

        $limit = (int) Tools::getValue($this->list_id.'_pagination', $limit);
        if (in_array($limit, $this->_pagination) && $limit != $this->_default_pagination) {
            $this->context->cookie->{$this->list_id.'_pagination'} = $limit;
        } else {
            unset($this->context->cookie->{$this->list_id.'_pagination'});
        }

        /* Check params validity */
        if (!Validate::isOrderBy($orderBy) || !Validate::isOrderWay($orderWay)
            || !is_numeric($start) || !is_numeric($limit)
            || !Validate::isUnsignedId($idLang)
        ) {
            throw new PrestaShopException('get list params is not valid');
        }

        if (!isset($this->fields_list[$orderBy]['order_key']) && isset($this->fields_list[$orderBy]['filter_key'])) {
            $this->fields_list[$orderBy]['order_key'] = $this->fields_list[$orderBy]['filter_key'];
        }

        if (isset($this->fields_list[$orderBy]) && isset($this->fields_list[$orderBy]['order_key'])) {
            $orderBy = $this->fields_list[$orderBy]['order_key'];
        }

        /* Determine offset from current page */
        $start = 0;
        if ((int) Tools::getValue('submitFilter'.$this->list_id)) {
            $start = ((int) Tools::getValue('submitFilter'.$this->list_id) - 1) * $limit;
        } elseif (empty($start) && isset($this->context->cookie->{$this->list_id.'_start'}) && Tools::isSubmit('export'.$this->table)) {
            $start = $this->context->cookie->{$this->list_id.'_start'};
        }

        // Either save or reset the offset in the cookie
        if ($start) {
            $this->context->cookie->{$this->list_id.'_start'} = $start;
        } elseif (isset($this->context->cookie->{$this->list_id.'_start'})) {
            unset($this->context->cookie->{$this->list_id.'_start'});
        }

        /* Cache */
        $this->_lang = (int) $idLang;
        $this->_orderBy = $orderBy;

        if (preg_match('/[.!]/', $orderBy)) {
            $orderBySplit = preg_split('/[.!]/', $orderBy);
            $orderBy = bqSQL($orderBySplit[0]).'.`'.bqSQL($orderBySplit[1]).'`';
        } elseif ($orderBy) {
            $orderBy = '`'.bqSQL($orderBy).'`';
        }

        $this->_orderWay = mb_strtoupper($orderWay);

        /* SQL table : orders, but class name is Order */
        $sqlTable = $this->table == 'order' ? 'orders' : $this->table;

        // Add SQL shop restriction
        $selectShop = $joinShop = $whereShop = '';
        if ($this->shopLinkType) {
            $selectShop = ', shop.name as shop_name ';
            $joinShop = ' LEFT JOIN '._DB_PREFIX_.$this->shopLinkType.' shop
							ON a.id_'.$this->shopLinkType.' = shop.id_'.$this->shopLinkType;
            $whereShop = Shop::addSqlRestriction($this->shopShareDatas, 'a');
        }

        if ($this->multishop_context && Shop::isTableAssociated($this->table) && !empty($this->className)) {
            if (Shop::getContext() != Shop::CONTEXT_ALL || !$this->context->employee->isSuperAdmin()) {
                $testJoin = !preg_match('#`?'.preg_quote(_DB_PREFIX_.$this->table.'_shop').'`? *sa#', $this->_join);
                if (Shop::isFeatureActive() && $testJoin && Shop::isTableAssociated($this->table)) {
                    $this->_where .= ' AND EXISTS (
						SELECT 1
						FROM `'._DB_PREFIX_.$this->table.'_shop` sa
						WHERE a.'.$this->identifier.' = sa.'.$this->identifier.' AND sa.id_shop IN ('.implode(', ', Shop::getContextListShopID()).')
					)';
                }
            }
        }

        /* Query in order to get results with all fields */
        $langJoin = '';
        if ($this->lang) {
            $langJoin = 'LEFT JOIN `'._DB_PREFIX_.$this->table.'_lang` b ON (b.`'.$this->identifier.'` = a.`'.$this->identifier.'` AND b.`id_lang` = '.(int) $idLang;
            if ($idLangShop) {
                if (!Shop::isFeatureActive()) {
                    $langJoin .= ' AND b.`id_shop` = '.(int) Configuration::get('PS_SHOP_DEFAULT');
                } elseif (Shop::getContext() == Shop::CONTEXT_SHOP) {
                    $langJoin .= ' AND b.`id_shop` = '.(int) $idLangShop;
                } else {
                    $langJoin .= ' AND b.`id_shop` = a.id_shop_default';
                }
            }
            $langJoin .= ')';
        }

        $havingClause = '';
        if (isset($this->_filterHaving) || isset($this->_having)) {
            $havingClause = ' HAVING ';
            if (isset($this->_filterHaving)) {
                $havingClause .= ltrim($this->_filterHaving, ' AND ');
            }
            if (!empty($this->_having)) {
                $havingClause .= $this->_having.' ';
            }
        }

        do {
            $this->_listsql = '';

            if ($this->explicitSelect) {
                foreach ($this->fields_list as $key => $arrayValue) {
                    // Add it only if it is not already in $this->_select
                    if (isset($this->_select) && preg_match('/[\s]`?'.preg_quote($key, '/').'`?\s*,/', $this->_select)) {
                        continue;
                    }

                    if (isset($arrayValue['filter_key'])) {
                        $this->_listsql .= str_replace('!', '.`', $arrayValue['filter_key']).'` AS `'.$key.'`, ';
                    } elseif ($key == 'id_'.$this->table) {
                        $this->_listsql .= 'a.`'.bqSQL($key).'`, ';
                    } elseif ($key != 'image' && !preg_match('/'.preg_quote($key, '/').'/i', $this->_select)) {
                        $this->_listsql .= '`'.bqSQL($key).'`, ';
                    }
                }
                $this->_listsql = rtrim(trim($this->_listsql), ',');
            } else {
                $this->_listsql .= ($this->lang ? 'b.*,' : '').' a.*';
            }

            $this->_listsql .= '
			'.(isset($this->_select) ? ', '.rtrim($this->_select, ', ') : '').$selectShop;

            $sqlFrom = '
			FROM `'._DB_PREFIX_.$sqlTable.'` a ';
            $sqlJoin = '
			'.$langJoin.'
			'.(isset($this->_join) ? $this->_join.' ' : '').'
			'.$joinShop;
            $sqlWhere = ' '.(isset($this->_where) ? $this->_where.' ' : '').($this->deleted ? 'AND a.`deleted` = 0 ' : '').
                (isset($this->_filter) ? $this->_filter : '').$whereShop.'
			'.(isset($this->_group) ? $this->_group.' ' : '').'
			'.$havingClause;
            $sqlOrderBy = ' ORDER BY '.((str_replace('`', '', $orderBy) == $this->identifier) ? 'a.' : '').$orderBy.' '.pSQL($orderWay).
                ($this->_tmpTableFilter ? ') tmpTable WHERE 1'.$this->_tmpTableFilter : '');
            $sqlLimit = ' '.(($useLimit === true) ? ' LIMIT '.(int) $start.', '.(int) $limit : '');

            if ($this->_use_found_rows || isset($this->_filterHaving) || isset($this->_having)) {
                $this->_listsql = 'SELECT SQL_CALC_FOUND_ROWS
								'.($this->_tmpTableFilter ? ' * FROM (SELECT ' : '').$this->_listsql.$sqlFrom.$sqlJoin.' WHERE 1 '.$sqlWhere.
                    $sqlOrderBy.$sqlLimit;
                $listCount = 'SELECT FOUND_ROWS() AS `'._DB_PREFIX_.$this->table.'`';
            } else {
                $this->_listsql = 'SELECT
								'.($this->_tmpTableFilter ? ' * FROM (SELECT ' : '').$this->_listsql.$sqlFrom.$sqlJoin.' WHERE 1 '.$sqlWhere.
                    $sqlOrderBy.$sqlLimit;
                $listCount = 'SELECT COUNT(*) AS `'._DB_PREFIX_.$this->table.'` '.$sqlFrom.$sqlJoin.' WHERE 1 '.$sqlWhere;
            }

            $this->_list = Db::getInstance()->executeS($this->_listsql, true, false);

            if ($this->_list === false) {
                $this->_list_error = Db::getInstance()->getMsgError();
                break;
            }

            $this->_listTotal = Db::getInstance()->getValue($listCount, false);

            if ($useLimit === true) {
                $start = (int) $start - (int) $limit;
                if ($start < 0) {
                    break;
                }
            } else {
                break;
            }
        } while (empty($this->_list));

        Hook::exec(
            'action'.$this->controller_name.'ListingResultsModifier', [
                'list'       => &$this->_list,
                'list_total' => &$this->_listTotal,
            ]
        );
    }

    public function processFilter()
    {
        Hook::exec('action'.$this->controller_name.'ListingFieldsModifier', ['fields' => &$this->fields_list]);

        $this->ensureListIdDefinition();

        $prefix = $this->getCookieFilterPrefix();

        if (isset($this->list_id)) {
            foreach ($_POST as $key => $value) {
                if ($value === '') {
                    unset($this->context->cookie->{$prefix.$key});
                } elseif (stripos($key, $this->list_id.'Filter_') === 0) {
                    $this->context->cookie->{$prefix.$key} = !is_array($value) ? $value : serialize($value);
                } elseif (stripos($key, 'submitFilter') === 0) {
                    $this->context->cookie->$key = !is_array($value) ? $value : serialize($value);
                }
            }

            foreach ($_GET as $key => $value) {
                if (stripos($key, $this->list_id.'Filter_') === 0) {
                    $this->context->cookie->{$prefix.$key} = !is_array($value) ? $value : serialize($value);
                } elseif (stripos($key, 'submitFilter') === 0) {
                    $this->context->cookie->$key = !is_array($value) ? $value : serialize($value);
                }
                if (stripos($key, $this->list_id.'Orderby') === 0 && Validate::isOrderBy($value)) {
                    if ($value === '' || $value == $this->_defaultOrderBy) {
                        unset($this->context->cookie->{$prefix.$key});
                    } else {
                        $this->context->cookie->{$prefix.$key} = $value;
                    }
                } elseif (stripos($key, $this->list_id.'Orderway') === 0 && Validate::isOrderWay($value)) {
                    if ($value === '' || $value == $this->_defaultOrderWay) {
                        unset($this->context->cookie->{$prefix.$key});
                    } else {
                        $this->context->cookie->{$prefix.$key} = $value;
                    }
                }
            }
        }

        $filters = $this->context->cookie->getFamily($prefix.$this->list_id.'Filter_');
        $definition = false;
        if (isset($this->className) && $this->className) {
            $definition = ObjectModel::getDefinition($this->className);
        }

        foreach ($filters as $key => $value) {
            if ($key === 'vatnumberscustomerFilter_v!vat_number') {
                $sqlFilter = &$this->_filter;

                $sqlFilter .= ' AND a.`id_customer` '.(!$value ? 'NOT' : '').' IN (SELECT `id_customer` FROM `'._DB_PREFIX_.'address` WHERE `vat_number` <> \'\') ';

                continue;
            }

            /* Extracting filters from $_POST on key filter_ */
            if ($value != null && !strncmp($key, $prefix.$this->list_id.'Filter_', 7 + mb_strlen($prefix.$this->list_id))) {
                $key = mb_substr($key, 7 + mb_strlen($prefix.$this->list_id));
                /* Table alias could be specified using a ! eg. alias!field */
                $tmpTab = explode('!', $key);
                $filter = count($tmpTab) > 1 ? $tmpTab[1] : $tmpTab[0];

                if ($field = $this->filterToField($key, $filter)) {
                    $type = (array_key_exists('filter_type', $field) ? $field['filter_type'] : (array_key_exists('type', $field) ? $field['type'] : false));
                    if (($type == 'date' || $type == 'datetime') && is_string($value)) {
                        $value = Tools::unSerialize($value);
                    }
                    $key = isset($tmpTab[1]) ? $tmpTab[0].'.`'.$tmpTab[1].'`' : '`'.$tmpTab[0].'`';

                    // Assignment by reference
                    if (array_key_exists('tmpTableFilter', $field)) {
                        $sqlFilter = &$this->_tmpTableFilter;
                    } elseif (array_key_exists('havingFilter', $field)) {
                        $sqlFilter = &$this->_filterHaving;
                    } else {
                        $sqlFilter = &$this->_filter;
                    }

                    /* Only for date filtering (from, to) */
                    if (is_array($value)) {
                        if (isset($value[0]) && !empty($value[0])) {
                            if (!Validate::isDate($value[0])) {
                                $this->errors[] = Tools::displayError('The \'From\' date format is invalid (YYYY-MM-DD)');
                            } else {
                                $sqlFilter .= ' AND '.pSQL($key).' >= \''.pSQL(Tools::dateFrom($value[0])).'\'';
                            }
                        }

                        if (isset($value[1]) && !empty($value[1])) {
                            if (!Validate::isDate($value[1])) {
                                $this->errors[] = Tools::displayError('The \'To\' date format is invalid (YYYY-MM-DD)');
                            } else {
                                $sqlFilter .= ' AND '.pSQL($key).' <= \''.pSQL(Tools::dateTo($value[1])).'\'';
                            }
                        }
                    } else {
                        $sqlFilter .= ' AND ';
                        $checkKey = ($key == $this->identifier || $key == '`'.$this->identifier.'`');
                        $alias = ($definition && !empty($definition['fields'][$filter]['shop'])) ? 'sa' : 'a';

                        if ($type == 'int' || $type == 'bool') {
                            $sqlFilter .= (($checkKey || $key == '`active`') ? $alias.'.' : '').pSQL($key).' = '.(int) $value.' ';
                        } elseif ($type == 'decimal') {
                            $sqlFilter .= ($checkKey ? $alias.'.' : '').pSQL($key).' = '.(float) $value.' ';
                        } elseif ($type == 'select') {
                            $sqlFilter .= ($checkKey ? $alias.'.' : '').pSQL($key).' = \''.pSQL($value).'\' ';
                        } elseif ($type == 'price') {
                            $value = (float) str_replace(',', '.', $value);
                            $sqlFilter .= ($checkKey ? $alias.'.' : '').pSQL($key).' = '.pSQL(trim($value)).' ';
                        } else {
                            $sqlFilter .= ($checkKey ? $alias.'.' : '').pSQL($key).' LIKE \'%'.pSQL(trim($value)).'%\' ';
                        }
                    }
                }
            }
        }
    }
}
