/**
 * Copyright (C) 2017-2018 thirty bees
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.md
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@thirtybees.com so we can send you a copy immediately.
 *
 * @author    thirty bees <contact@thirtybees.com>
 * @copyright 2017-2018 thirty bees
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */
(function () {
  var setXhr = null;
  window.vatNumberModule = window.vatNumberModule || {};

  window.vatNumberModule.listValidateClickHandler = function (event) {
    var $elem = $(event.target).closest('span.badge');
    if (!$elem.data('id-customer')) {
      return;
    }

    var idCustomer = $elem.data('id-customer');
    var vatNumber = $elem.data('vat-number');

    $.ajax({
      url: window.vat_number_ajax,
      dataType: 'json',
      data: {
        idCustomer: idCustomer,
        vatNumber: vatNumber,
        ajax: 1,
        action: 'ValidateVatNumber',
      },
      success: function (response) {
        if (response && response.success) {
          $elem.hide();
          if (response.check && response.check.valid) {
            $('#vat-number-valid-' + idCustomer).show();
          } else {
            $('#vat-number-invalid-' + idCustomer).show();
            window.showErrorMessage('Invalid VAT Number');
          }
          if (typeof response.check.traderName === 'string') {
            $('#vat-company-' + idCustomer).text(response.check.traderName);
          }
          if (typeof response.check.traderAddress === 'string') {
            $('#vat-address-' + idCustomer).text(response.check.traderAddress);
          }
        } else if (response && response.message) {
          window.showErrorMessage(response.message);
        }
      },
    });
  };

  window.vatNumberModule.setNewVatNumber = function (event) {
    var $elem = $(event.target).closest('span');
    console.log($elem);
    if (!$elem.data('id-customer')) {
      return;
    }

    var idCustomer = $elem.data('id-customer');
    var vatNumber = $('#vat-number-input-' + idCustomer).val();

    if (setXhr && typeof setXhr.abort === 'function') {
      setXhr.abort();
    }

    setXhr = $.ajax({
      url: window.vat_number_ajax,
      dataType: 'json',
      data: {
        idCustomer: idCustomer,
        vatNumber: vatNumber,
        ajax: 1,
        action: 'ValidateAndSetVatNumber',
      },
      success: function (response) {
        if (response && response.success) {
          $elem.hide();
          if (response.check && response.check.valid) {
            $('#vat-number-valid-' + idCustomer).show();
            $('#vat-number-valid-' + idCustomer).parent().show();
            $('#vat-number-input-' + idCustomer).parent().hide();
            $('#btn-group-' + idCustomer).show();

            if (response.vatNumber) {
              $('#vat-number-valid-' + idCustomer)
                .find('span.vat-number')
                .text(vatNumber)
                .data('vat-number', vatNumber);
              $('#vat-number-manual-' + idCustomer)
                .find('span.vat-number')
                .text(vatNumber)
                .data('vat-number', vatNumber);
              $('#vat-number-invalid-' + idCustomer)
                .find('span.vat-number')
                .text(vatNumber)
                .data('vat-number', vatNumber);
            }
          } else {
            $('#vat-number-invalid-' + idCustomer).show();
            window.showErrorMessage('Invalid VAT Number');
          }
          if (typeof response.check.traderName === 'string') {
            $('#vat-company-' + idCustomer).text(response.check.traderName);
          }
          if (typeof response.check.traderAddress === 'string') {
            $('#vat-address-' + idCustomer).text(response.check.traderAddress);
          }
        } else if (response && response.message) {
          window.showErrorMessage(response.message);
        }
      },
    });
  };

  window.vatNumberModule.toggleInput = function (event) {
    var $elem = $(event.target).closest('button');
    var idCustomer = $elem.data('id-customer');
    if ($('#vat-number-input-' + idCustomer).is(':visible')) {
      $('#vat-number-valid-' + idCustomer).parent().show();
      $('#vat-number-input-' + idCustomer).parent().hide();
      $('#vat-number-set-' + idCustomer).hide();

      $('#btn-group-' + idCustomer).show();
    } else {
      $('#vat-number-valid-' + idCustomer).parent().hide();
      $('#vat-number-input-' + idCustomer).parent().show();
      $('#vat-number-set-' + idCustomer).show();
      $('#btn-group-' + idCustomer).hide();
    }
  };

  function init() {
    if (typeof $ === 'undefined') {
      setTimeout(init, 100);

      return;
    }
  }

  init();
}());
